#version 430 core

layout (vertices = 4) out;

// frustum culling works, but does not have a very pronounced effect on framerate
#define DO_FRUSTUM_CULLING

in  vec2 vs_uvcoords[];  // input  uv coordinates from vs
out vec2 tcs_uvcoords[]; // output uv coordinates from tcs

uniform mat4 matModel; // model matrix
uniform mat4 matProjView; // proj*view matrix
uniform int min_subdivisions, max_subdivisions;
uniform float subdivision_factor;
uniform float min_height;
uniform float max_height;
uniform sampler2D texHeightmap; // heightmap

void main() {
	// TODO: tesselate according to distance to player; or screen size of the patch
	float outer = max_subdivisions, inner = min_subdivisions;
	if (gl_InvocationID == 0) {
		// calculate length of quad sides

		// TODO: consider height-variation in quad for tesselating? - mark "difficult" areas in separate texture / other color channel?

		// gl_InvocationID: 0 top left, 1 top right, 2 bottom left, 3 bottom right
		// Outer levels: 0 left, 1 bottom, 2 right, 3 top

		// get the corner vertices of the patch
		vec4 v0 = gl_in[0].gl_Position;
		vec4 v1 = gl_in[1].gl_Position;
		vec4 v2 = gl_in[2].gl_Position;
		vec4 v3 = gl_in[3].gl_Position;

		// set the height offset before transforming!
		v0.y = min_height + texture(texHeightmap, vs_uvcoords[0]).r * (max_height - min_height);
		v1.y = min_height + texture(texHeightmap, vs_uvcoords[1]).r * (max_height - min_height);
		v2.y = min_height + texture(texHeightmap, vs_uvcoords[2]).r * (max_height - min_height);
		v3.y = min_height + texture(texHeightmap, vs_uvcoords[3]).r * (max_height - min_height);

		// transform to NDC space (m,v,p,w-div)
		mat4 mvp = matProjView * matModel;
		v0 = mvp * v0;
		v1 = mvp * v1;
		v2 = mvp * v2;
		v3 = mvp * v3;

		// not necessary to render stuff behind (i.e. in "front" of the screen)
		if (v0.z < 0 && v1.z < 0 && v2.z < 0 && v3.z < 0) {
			gl_TessLevelOuter[0] = 0;
			gl_TessLevelOuter[1] = 0;
			gl_TessLevelOuter[2] = 0;
			gl_TessLevelOuter[3] = 0;
			gl_TessLevelInner[0] = 0;
			gl_TessLevelInner[1] = 0;
		} else {
			// w-divide
			v0 = v0 / v0.w;
			v1 = v1 / v1.w;
			v2 = v2 / v2.w;
			v3 = v3 / v3.w;

			#ifdef DO_FRUSTUM_CULLING
				bool cull;
				cull =  (v0.x < -1 && v1.x < -1 && v2.x < -1 && v3.x < -1)
				     || (v0.x >  1 && v1.x >  1 && v2.x >  1 && v3.x >  1)
				     || (v0.y < -1 && v1.y < -1 && v2.y < -1 && v3.y < -1)
				     || (v0.y >  1 && v1.y >  1 && v2.y >  1 && v3.y >  1)
				     || (v0.z >  1 && v1.z >  1 && v2.z >  1 && v3.z >  1); // case all z < -1 obsolete, already checked against 0 (?)
			#else
				const bool cull = false;
			#endif

			if (cull) {
				gl_TessLevelOuter[0] = 0;
				gl_TessLevelOuter[1] = 0;
				gl_TessLevelOuter[2] = 0;
				gl_TessLevelOuter[3] = 0;
				gl_TessLevelInner[0] = 0;
				gl_TessLevelInner[1] = 0;
			} else {
				// calc lengths in NDC space
				float len[4] = {abs(length(v0.xy-v2.xy)), // left
								abs(length(v2.xy-v3.xy)), // bottom
								abs(length(v1.xy-v3.xy)), // right
								abs(length(v0.xy-v1.xy))  // top
								};

				gl_TessLevelOuter[0] = min(max_subdivisions, max(min_subdivisions, subdivision_factor * len[0]));
				gl_TessLevelOuter[1] = min(max_subdivisions, max(min_subdivisions, subdivision_factor * len[1]));
				gl_TessLevelOuter[2] = min(max_subdivisions, max(min_subdivisions, subdivision_factor * len[2]));
				gl_TessLevelOuter[3] = min(max_subdivisions, max(min_subdivisions, subdivision_factor * len[3]));
				gl_TessLevelInner[0] = (gl_TessLevelOuter[1] + gl_TessLevelOuter[3]) / 2; // horizontal
				gl_TessLevelInner[1] = (gl_TessLevelOuter[0] + gl_TessLevelOuter[2]) / 2; // vertical
			}
		}
	}
	// pass through position and uv coordinates from vertex shader
	gl_out[gl_InvocationID].gl_Position = gl_in[gl_InvocationID].gl_Position;
	tcs_uvcoords[gl_InvocationID] = vs_uvcoords[gl_InvocationID];
}
