#version 430 core

//layout (quads, fractional_odd_spacing) in;
layout (quads) in;

in  vec2 tcs_uvcoords[]; // input  uv coordinates from tcs (4)
out vec2 tes_uvcoords;	 // output uv coordinates from tes (1 per invocation)
out vec3 worldpos3;		 // world coordinates
out vec3 frag_campos3;   // fragment in camera coordinates

uniform mat4 matModel; // model matrix
uniform mat4 matProjView; // proj*view matrix
uniform float min_height;
uniform float max_height;
uniform sampler2D texHeightmap; // heightmap


void main() {
	// calculate vertex coordinates for current vertex
    vec4 p1 = mix(gl_in[0].gl_Position, gl_in[1].gl_Position, gl_TessCoord.x);
    vec4 p2 = mix(gl_in[2].gl_Position, gl_in[3].gl_Position, gl_TessCoord.x);
    vec4 p  = mix(p2, p1, gl_TessCoord.y);

	// calculate uv coordinates for current vertex
    vec2 uv1 = mix(tcs_uvcoords[0], tcs_uvcoords[1], gl_TessCoord.x);
    vec2 uv2 = mix(tcs_uvcoords[2], tcs_uvcoords[3], gl_TessCoord.x);
    vec2 uv  = mix(uv2, uv1, gl_TessCoord.y);

	// offset y coordinate from heightmap
    p.y = min_height + texture(texHeightmap, uv).r * (max_height - min_height);

	vec4 worldpos4 = matModel * p;
	worldpos3 = vec3(worldpos4);

    gl_Position = matProjView * worldpos4;
	tes_uvcoords = uv;
}
