#version 430 core

layout (location=0) out vec4 fragColor;		// shader output

// fragment position & normal & texture coords
in vec3 position;	// in world space
in vec3 normal;		// in world space
in vec2 uvcoords;

layout (location=100) uniform sampler2D texDiffColor; // noise

//uniform sampler2D lightMap;
//uniform vec2 invWorldSize; // 1/worldsize in OGL units
uniform vec3 playerPos;
uniform vec3 lightDirNormed; // direction TO light; must be normalized!
uniform bool playerShadow;

uniform float timeSeconds;
uniform float aspectRatio;

uniform float globalBrightnessMultiplier = 1.0;


float time = timeSeconds * 0.1f;

const float speed1 = 0.3; //.6;		// primary flow
const float speed2 = 0.0; //1.9;	// perceived flow
const float speed3 = 1.0;           // displacement field
const float advection = .77;		// advection factor for blending (.5 - .95)

mat2 rotMat2D(in float theta) {
  float c = cos(theta);
  float s = sin(theta);
  return mat2(c,-s,s,c);
}

float noise(in vec2 x) {
  return texture(texDiffColor, x*.01).x;
}

vec2 gradient(vec2 p) {
	const float epsilon = .09;
	float gradx = noise(vec2(p.x+epsilon,p.y))-noise(vec2(p.x-epsilon,p.y));
	float grady = noise(vec2(p.x,p.y+epsilon))-noise(vec2(p.x,p.y-epsilon));
	return vec2(gradx,grady);
}

float flow(in vec2 p) {
	float z=2.;
	float rz = 0.;
	vec2 bp = p;
	for (float i= 1.;i < 7.;i++ )
	{
		p  += time * speed1; //primary flow speed
		bp += time * speed2; //secondary flow speed (speed of the perceived flow)
		vec2 gr = gradient(i*p*.34+time*speed3); //displacement field
		gr*=rotMat2D(time*6.-(0.05*p.x+0.03*p.y)*40.); //rotation of the displacement field
		p += gr*.5; //displace the system
		rz+= (sin(noise(p)*7.)*0.5+0.5)/z; //add noise octave
		p = mix(bp,p,advection); // blend displaced system with base system
		z *= 1.4; //intensity scaling
		p *= 2.; bp *= 1.9; //octave scaling
	}
	return rz;	
}

float mockupShadow() {
	// mock-up-shadow
	float illum = 1.0;
	if (playerShadow) {
		vec3 vPlayer = playerPos - position;
		float playerDistance = length(vPlayer);
		// calc angle between direction to player and direction to light
		float cosAngle = dot(normalize(vPlayer), lightDirNormed);

		if (cosAngle > 0) {
			float angle = acos(cosAngle);
			float d = abs(playerDistance * tan(angle));

			const float d_out = 2.5; // approx. player size
			const float d_in  = 1.5; // approx. player size
			const float il_min = .3;
			if (d < d_out) { 
				illum = il_min;
				if (d > d_in) {
					illum = il_min + (1-il_min)*smoothstep(d_in, d_out, d);
				}
			}
		}
	}

	return illum;
}


void main()
{
	// todo: mirroring / tiling?

	vec2 p = fract(uvcoords) - vec2(.5,.5);
	p.x *= aspectRatio; // aspect ratio ?
	p*= 3.;
	float rz = flow(p);
	
	vec3 col = vec3(.2,0.07,0.01)/rz;
	col=pow(col,vec3(1.4));

//	// use *some* light from lightMap - does not! look good for lava
//	const float terrainLight = .5f;
//	vec2 lightUV = vec2(position.x, -position.z) * invWorldSize + vec2(.5,.5); // flip Y !
//	float light = texture(lightMap, lightUV).r;
//	col = (1.0 - terrainLight) * col + terrainLight * light;

	// player mockup shadow
	col *= mockupShadow();

	fragColor = vec4(col * globalBrightnessMultiplier, 1.0);

}

