#version 430 core

layout (points) in;
layout (points, max_vertices=30) out;

//#define PARTICLE_DEBUGGING

// input from vertex shader
in vec4 vs_position_type[];
in vec4 vs_velocity_ttl[];
in vec4 vs_color[];

// output to transform feedback buffer
out vec4 out_position_type;
out vec4 out_velocity_ttl;
out vec4 out_color;

// control uniforms
//uniform bool emitter_active;
uniform vec3 emitter_position;
uniform vec3 emitter_dir;
uniform vec3 emitter_velocity;
uniform int emitter_numParticles;
uniform float time;
uniform float deltaT; // time since last update
uniform sampler1D texRandom;
uniform mat3 coneRotMat;
uniform float particle_time = 5000.0; // millis
uniform float particle_random_speed;
uniform float emitter_time = 1.0;       // millis

#ifdef PARTICLE_DEBUGGING
uniform float debug_cg;
uniform float debug_cb;
#endif


#define PARTICLE_TYPE_EMITTER  0.0f
#define PARTICLE_TYPE_PARTICLE 1.0f

vec3 randomVel(float seed) { // already oriented towards emitter_dir
	//return normalize(texture(texRandom, seed).xyz - vec3(.5));
	vec4 rand = texture(texRandom, seed);
	return coneRotMat * rand.xyz * rand.w;
}

void main() {
	float dtMs = deltaT * 1000.0f;
	float ttl = vs_velocity_ttl[0].w - dtMs; // calc new time-to-live (millseconds)

	if (vs_position_type[0].w == PARTICLE_TYPE_EMITTER) {
		// output the emitter first! otherwise it may get pushed back in the buffer and eventually dropped!

		// pass through the emitter itself (positition, velocity are not used anyway)
		out_position_type    = vs_position_type[0];
		out_velocity_ttl.xyz = vs_velocity_ttl [0].xyz;
		out_velocity_ttl.w   = ttl <= 0 ? emitter_time : ttl; // reset emitter time if ttl is <= 0
		out_color            = vs_color[0];
		#ifdef PARTICLE_DEBUGGING
			out_color.r = vs_color[0].r + 5.0f; // DEBUG ONLY! - REMOVE
		#endif
		EmitVertex(); EndPrimitive();

		if (ttl <= 0) {
			// emit new particles (if emitter is active)
			for (int i = 0; i < emitter_numParticles; i++) {
				out_position_type.xyz = emitter_position;
				out_position_type.w   = PARTICLE_TYPE_PARTICLE;
				//out_velocity_ttl.xyz  = emitter_velocity + .5 * randomDir(fract(time)); // TODO - only downwards
				out_velocity_ttl.xyz  = emitter_velocity + particle_random_speed * randomVel(fract(time + i * .01)); // TODO - only downwards
				out_velocity_ttl.w    = particle_time;
				out_color             = vs_color[0];
				#ifdef PARTICLE_DEBUGGING
					out_color.r = vs_color[0].r + i; // DEBUG ONLY! - REMOVE
					out_color.g = debug_cg;
					out_color.b = debug_cb;
				#endif
				EmitVertex(); EndPrimitive();
			}
		}
	} else {
		if (ttl > 0) {
			// live on.. update position
			out_position_type.xyz = vs_position_type[0].xyz + deltaT * vs_velocity_ttl[0].xyz;
			out_position_type.w   = vs_position_type[0].w;
			out_velocity_ttl.xyz  = vs_velocity_ttl [0].xyz;
			out_velocity_ttl.w    = ttl;
			out_color             = vs_color[0];
			EmitVertex(); EndPrimitive();
		}
		// else die
	}
}

