#version 430 core

// attributes
layout(location = 0) in vec3 position_in;
layout(location = 1) in vec3 normal_in;
layout(location = 2) in vec2 uvcoords_in;
layout(location = 3) in vec3 tangent_in;

// pass through uv coordinates
out vec2 uvcoords;

out vec3 vLight; // vector from vertex TO light
out vec3 vToEye; // vector from vertex TO eye
out flat vec3 light_amb;
out flat vec3 light_diff;
out flat vec3 light_spec;

// uniforms
layout(location = 0) uniform mat4 matModel; // model matrix
uniform mat4 matProjView; // proj*view matrix

// stuff that is usually in lighting.h
struct BasicLight {
	int  enabled;
	vec3 color_amb, color_diff, color_spec;
};

struct DirectionalLight {
	BasicLight light;
	vec3 direction;
};

layout (location=8)  uniform vec3 eye;				// eye position (camera)
layout (location=20) uniform DirectionalLight dirLight[1]; // 20..23  (4 per light)	
// -- end -- stuff that is usually in lighting.h

void main()
{
	
	vec3 position = vec3(matModel * vec4(position_in, 1.0));

	mat3 normMat = transpose(inverse(mat3(matModel)));
	vec3 normal  = normalize(normMat * normal_in);
	vec3 tangent = normalize(normMat * tangent_in);
	vec3 bitangent = cross(normal, tangent);
	// we *could* renormalize the normal; not for now

	mat3 tbnMat = transpose(mat3(tangent, bitangent, normal));

	// pass light direction and view direction in TANGENT SPACE to the fragment shader
	vLight = tbnMat * (-dirLight[0].direction);
	vToEye = tbnMat * (eye - position);

	light_amb  = dirLight[0].light.color_amb;
	light_diff = dirLight[0].light.color_diff;
	light_spec = dirLight[0].light.color_spec;

	gl_Position = matProjView * vec4(position, 1.0f);
	uvcoords = uvcoords_in;
}
