#version 430 core

// TODO: move tangent space transform to vertex shader

// stuff usually in lighting.h
struct Material {
	vec3 ambient, diffuse, specular;
	float shininess;
};

layout (location=0) out vec4 fragColor;		// shader output

// fragment position & normal & texture coords
in vec2 uvcoords;

in vec3 vLight; // vector from vertex TO light in tangent space
in vec3 vToEye; // vector from vertex TO eye   in tangent space
in flat vec3 light_amb;
in flat vec3 light_diff;
in flat vec3 light_spec;


layout (location=11) uniform Material material;		// material uniform Phong (loc. 11-15)

uniform sampler2D texHeightmap; // NORMAL map

uniform float globalBrightnessMultiplier = 1.0;

void main()
{

	vec3 vNorm = normalize(texture(texHeightmap, uvcoords).rgb * 2.0 - 1.0); // normal from normal map; in tangent space
	vec3 vView = normalize(vToEye);	// calc. normalized view vector (from position to eye);             in tangent space

	// set mat_color - the material.color - to texture values, if a texture is assigned
	Material final_material = material;

	// phong lighting calculation (usually in "lighting.h")
	vec3 vRefl = reflect(-vLight, vNorm); // reflected light beam vector
	float diff_factor = max(dot(vLight, vNorm), 0.0f); // clamp to 0
	float spec_factor = max(dot(vRefl,  vView), 0.0f); // clamp to 0
	vec3 res_d = final_material.diffuse  * diff_factor * light_diff;
	vec3 res_s;
	if (final_material.shininess == 0) {
		res_s = vec3(0); // no shininess; in some obj. files...
	} else {
		res_s = final_material.specular * pow(spec_factor, final_material.shininess) * light_spec;
	}

	vec3 finalColor = light_amb * final_material.ambient + res_d + res_s;
	fragColor = vec4(finalColor * globalBrightnessMultiplier, 1.0);
}
