#version 430 core

layout (location=0) out vec4 fragColor;		// shader output

// fragment position & normal & texture coords
in vec3 position;	// in world space
in vec3 normal;		// in world space
in vec2 uvcoords;

layout (location=100) uniform sampler2D texDiffColor; // noise

//uniform sampler2D lightMap;
//uniform vec2 invWorldSize; // 1/worldsize in OGL units

uniform vec3 playerPos;
uniform vec3 lightDirNormed; // direction TO light; must be normalized!
uniform bool playerShadow;
uniform vec3 eye;			 // eye position (camera)

uniform float globalBrightnessMultiplier = 1.0;

const float water_alpha = 0.7;
const vec3  water_color = vec3(0.1,0.3,0.3);
const vec3  water_spec  = vec3(1.0,1.0,1.0);
const float water_shiny = 30.0;

float mockupShadow() {
	// mock-up-shadow
	float illum = 1.0;
	if (playerShadow) {
		vec3 vPlayer = playerPos - position;
		float playerDistance = length(vPlayer);
		// calc angle between direction to player and direction to light
		float cosAngle = dot(normalize(vPlayer), lightDirNormed);

		if (cosAngle > 0) {
			float angle = acos(cosAngle);
			float d = abs(playerDistance * tan(angle));

			const float d_out = 2.5; // approx. player size
			const float d_in  = 1.5; // approx. player size
			const float il_min = .3;
			if (d < d_out) { 
				illum = il_min;
				if (d > d_in) {
					illum = il_min + (1-il_min)*smoothstep(d_in, d_out, d);
				}
			}
		}
	}

	return illum * globalBrightnessMultiplier;
}

vec3 specular() {
	vec3 vNorm = normalize(normal);	// renormalize the normal
	vec3 vView = normalize(eye - position);	// calc. normalized view vector (from position to eye)
	vec3 vRefl = reflect(-lightDirNormed, vNorm); // reflected light beam vector
	float spec_factor = max(dot(vRefl,  vView), 0.0f); // clamp to 0
	return water_spec * pow(spec_factor, water_shiny);
}

void main()
{
	vec3 col = water_color + specular();

//	// use *some* light from lightMap - does not look so great (but ok-ish) for water
//	const float terrainLight = .5f;
//	vec2 lightUV = vec2(position.x, -position.z) * invWorldSize + vec2(.5,.5); // flip Y !
//	float light = texture(lightMap, lightUV).r;
//	col = (1.0 - terrainLight) * col + terrainLight * light;
//
	

	// player mockup shadow
	float illum = mockupShadow();
	col *= illum;

	fragColor = vec4(col,water_alpha); // transparent! render with blending!
}

