#version 430 core

layout (location=0) out vec4 fragColor;		// shader output

// fragment position & normal & texture coords
in vec3 position;	// in world space
//in vec3 normal;		// in world space
in vec2 uvcoords;

uniform sampler2D normalMap;
uniform sampler2D displacementMap;
uniform float movement;

uniform vec3 playerPos;
uniform vec3 lightDirNormed; // direction TO light; must be normalized!
uniform bool playerShadow;
uniform vec3 eye;			 // eye position (camera)

uniform float globalBrightnessMultiplier = 1.0;

const float water_alpha = 0.7;
const vec3  water_color = vec3(0.3,0.5,0.9);
const vec3  water_spec  = vec3(1.0,1.0,1.0);
const float water_shiny = 30.0;

vec3 normal; // water normal equals unitY; modified normal taken from normal map

const float displacementStrength1 = .1;
const float displacementStrength2 = 1.0;

float mockupShadow() {
	// mock-up-shadow
	float illum = 1.0;
	if (playerShadow) {
		vec3 vPlayer = playerPos - position;
		float playerDistance = length(vPlayer);
		// calc angle between direction to player and direction to light
		float cosAngle = dot(normalize(vPlayer), lightDirNormed);

		if (cosAngle > 0) {
			float angle = acos(cosAngle);
			float d = abs(playerDistance * tan(angle));

			const float d_out = 2.5; // approx. player size
			const float d_in  = 1.5; // approx. player size
			const float il_min = .3;
			if (d < d_out) { 
				illum = il_min;
				if (d > d_in) {
					illum = il_min + (1-il_min)*smoothstep(d_in, d_out, d);
				}
			}
		}
	}

	return illum * globalBrightnessMultiplier;
}

vec3 specular() {
	vec3 vNorm = normalize(normal);	// renormalize the normal
	vec3 vView = normalize(eye - position);	// calc. normalized view vector (from position to eye)
	vec3 vRefl = reflect(-lightDirNormed, vNorm); // reflected light beam vector
	float spec_factor = max(dot(vRefl,  vView), 0.0f); // clamp to 0
	return water_spec * pow(spec_factor, water_shiny);
}

void main() {
	// variant 1
	//vec2 disp = texture(displacementMap, vec2(uvcoords.x + movement, uvcoords.y + movement)).rg;
	//vec2 newUV = uvcoords + disp * displacementStrength1;

	// variant 2
	vec2 disp  = texture(displacementMap, vec2(uvcoords.x + movement, uvcoords.y)).rg * displacementStrength1; // first offset x only
	vec2 newUV = uvcoords + vec2(disp.x, disp.y + movement) * displacementStrength2; // use displaced uv to displace uv, now with y offset

	// both variants are ok, variant 2 looking slightly better

	vec3 nmCol = texture(normalMap, newUV).rgb;
	normal = vec3(nmCol.r * 2.0 - 1.0, nmCol.b, nmCol.g * 2.0 - 1.0); // yes, the normal map is set up in such a way... (b = unscaled up)
	// normal is not normalized, this is done in specular()

	vec3 col = water_color + specular();

	// player mockup shadow
	float illum = mockupShadow();
	col *= illum;

	fragColor = vec4(col,water_alpha); // transparent! render with blending!

}

